<?php

final class FinalClass1
{

}

final /* comment */ class FinalClass2
{

}

class NonFinalClass1
{

}

class NonFinalClass2
{

}

class NonFinalClass3
{

}

abstract class AbstractClass1
{

}

abstract class AbstractClass2
{

}

abstract class AbstractClass3
{

}

abstract /* a comment */ class AbstractClass4
{

}
