<?php

class ClassConstants
{
    const IMPLICITLY_PUBLIC_CONSTANT = 'implicitly public';
    public const EXPLICITLY_PUBLIC_CONSTANT = 'explicitly public';
    private /* a comment might be here */ const COMMENT_IN_DEFINTION = 'comment in definition';
    private const PRIVATE_CONSTANT = 'private';
    protected const PROTECTED_CONSTANT = 'protected';
}
