<?php
class ATest extends PHPUnit\Framework\TestCase
{
    public function testFoo()
    {
    }

    /**
     * @test
     * @dataProvider barProvider
     */
    public function bar()
    {
    }

    protected function doSomething()
    {
    }

    public function barProvider()
    {
    }
}
