<?php

class MethodsPerClass1
{
    // no methods
}

class MethodsPerClass2
{
    public function method1()
    {
    }

    public function method2()
    {
    }
}

class MethodsPerClass3
{
    public function method1()
    {
    }

    public function method2()
    {
    }

    public function method3()
    {
    }

    public function method4()
    {
    }
}
